% CSDGrad - Complex step derivative gradient of a function.
%
% Calculates the gradient of a function using a complex step derivative.
% Note: functions must not use Matlab's ' operator which performs a
% transpose and complex complement.  Instead, use .' which only performs a
% transpose.
%
% CSD seems to work better with a smaller step size than finite difference.
%
% modification history
% --------------------
% 16jul2009 bss written.
%

function [ vCSDGrad ] = CSDGrad( func, x0, dwStep ) 

  nParams = length( x0 ) ;
  
  vCSDGrad = zeros( nParams, 1 ) ;

	% Calculate FD derivative by percentage
  %   fdTol = 0.1 ;  % step-size as a percentage of xFree.
  %   dx = fdTol * x0 ;
  
  if nargin < 3
    dx = 1e-5 ;
  else
    dx = dwStep ;
  end

  xPlus = x0 + 1i * dx ;

  for ix = 1 : nParams
    x1 = x0 ;
    x1( ix ) = xPlus( ix ) ;
    [ fval ] = func( x1 ) ;
    vCSDGrad( ix ) = imag( fval / dx ) ;
  end
    
  