% CSDHessian - calculates Hessian using complex step differentiation
%
% Algorithm:  Calculate Complex Step Differentiation Hessian of a scalar
% function.
%
% modification history
% --------------------
% 16jul2009 bss written.
%

function [ fdHess ] = CSDHessian( func, x0, dwStep )

nParams = length( x0 ) ;
fdHess  = zeros( nParams ) ;

for ix = 1 : nParams
  
  xImagStep       = x0 ;
  xImagStep( ix ) = x0( ix ) + 1i * dwStep ;
  
  for jx = ix : nParams
    
    xLeft        = xImagStep ;
    xLeft( jx )  = xLeft( jx ) - dwStep ;
    
    xRight       = xImagStep ;
    xRight( jx ) = xRight( jx ) + dwStep ;
    
    vLeftGrad  = func( xLeft ) ;
                        
    vRightGrad = func( xRight ) ;
                        
    fdHess( ix, jx ) = imag( ( vRightGrad - vLeftGrad ) / ( 2 * dwStep^2 ) ) ;                    
    fdHess( jx, ix ) = fdHess( ix, jx ) ;
  end
  
end