% FDGrad - Finite difference gradient of a function.
%
% Calculates a basic one- or two-sided finite difference gradient of a function
%
% modification history
% --------------------
% 08jul2009 bss written.
%

function [ vFDGrad ] = FDGrad( func, x0, dwStep, bOneSided ) 

  nParams = length( x0 ) ;
  
  vFDGrad = zeros( nParams, 1 ) ;

	% Calculate FD derivative by percentage
  %   fdTol = 0.1 ;  % step-size as a percentage of xFree.
  %   dx = fdTol * x0 ;
  
  if nargin < 3
    dx = 1e-5 ;
  else
    dx = dwStep ;
  end
  
  if nargin < 4 
    bOneSided = 1 ;
  end
  
  if bOneSided
  
    [ fval0 ] = func( x0 ) ;
    
    xPlus = x0 + dx ;

    for ix = 1 : nParams
      x1 = x0 ;
      x1( ix ) = xPlus( ix ) ;
      [ fval1 ] = func( x1 ) ;
      vFDGrad( ix ) = ( fval1 - fval0 ) / ( x1( ix ) - x0( ix ) ) ;
    end
    
  else

    xPlus  = x0 + dx ;
    xMinus = x0 - dx ;

    for ix = 1 : nParams
      xHi = x0 ;
      xHi( ix ) = xPlus( ix ) ;
      
      xLo = x0 ;
      xLo( ix ) = xMinus( ix ) ;
      
      [ fvalHi ] = func( xHi ) ;
      [ fvalLo ] = func( xLo ) ;
      vFDGrad( ix ) = ( fvalHi - fvalLo ) ./ ( xHi( ix ) - xLo( ix ) ) ;
    end
        
  end